import logging
from pathlib import Path
from typing import Generator

import pytest
from pytest_docker.plugin import Services

from tika_client.client import TikaClient

logger = logging.getLogger("tika-client.tests")


@pytest.fixture(scope="session")
def docker_compose_file() -> Path:
    return Path(__file__).parent / "docker" / "docker-compose.ci-test.yml"


@pytest.fixture(scope="session")
def tika_host(docker_services: Services, docker_ip: str) -> str:
    def is_responsive(url):
        import httpx

        try:
            response = httpx.get(url)
        except httpx.HTTPError:
            logger.exception("Error connecting to service")
            return False
        else:
            return response.status_code == httpx.codes.OK

    url = f"http://{docker_ip}:{docker_services.port_for('tika', 9998)}"

    docker_services.wait_until_responsive(
        timeout=30.0,
        pause=1,
        check=lambda: is_responsive(url),
    )
    return url


@pytest.fixture(scope="session")
def samples_dir() -> Path:
    return Path(__file__).parent.resolve() / "samples"


@pytest.fixture(scope="session")
def sample_libre_office_writer_file(samples_dir: Path) -> Path:
    return samples_dir / "sample-libre-office.odt"


@pytest.fixture(scope="session")
def sample_google_docs_to_libre_office_writer_file(samples_dir: Path) -> Path:
    return samples_dir / "sample.odt"


@pytest.fixture(scope="session")
def sample_google_docs_to_docx_file(samples_dir: Path) -> Path:
    return samples_dir / "sample.docx"


@pytest.fixture(scope="session")
def sample_docx_file(samples_dir: Path) -> Path:
    return samples_dir / "microsoft-sample.docx"


@pytest.fixture(scope="session")
def sample_doc_file(samples_dir: Path) -> Path:
    return samples_dir / "sample.doc"


@pytest.fixture(scope="session")
def sample_html_file(samples_dir: Path) -> Path:
    return samples_dir / "sample.html"


@pytest.fixture(scope="session")
def sample_office_doc_with_images_file(samples_dir: Path) -> Path:
    return samples_dir / "test-document-images.odt"


@pytest.fixture(scope="session")
def sample_jpeg_file(samples_dir: Path) -> Path:
    return samples_dir / "sample.jpg"


@pytest.fixture(scope="session")
def sample_png_file(samples_dir: Path) -> Path:
    return samples_dir / "sample.png"


@pytest.fixture(scope="session")
def sample_ods_file(samples_dir: Path) -> Path:
    return samples_dir / "sample-spreadsheet.ods"


@pytest.fixture(scope="session")
def sample_xlsx_file(samples_dir: Path) -> Path:
    return samples_dir / "sample-spreadsheet.xlsx"


@pytest.fixture
def tika_client(tika_host: str) -> Generator[TikaClient, None, None]:
    with TikaClient(tika_url=tika_host, log_level=logging.INFO) as client:
        yield client


@pytest.fixture
def tika_client_compressed(tika_host: str) -> Generator[TikaClient, None, None]:
    with TikaClient(tika_url=tika_host, log_level=logging.INFO, compress=True) as client:
        yield client



from pathlib import Path

import magic

from tika_client.client import TikaClient


class TestRecursiveMetadataResource:
    def test_r_metadata_from_docx(self, tika_client: TikaClient, sample_google_docs_to_docx_file: Path):
        documents = tika_client.rmeta.as_html.from_file(
            sample_google_docs_to_docx_file,
            magic.from_file(str(sample_google_docs_to_docx_file), mime=True),
        )

        assert len(documents) == 1
        document = documents[0]

        assert document.type == "application/vnd.openxmlformats-officedocument.wordprocessingml.document"
        assert document.content is not None
        assert "<body><p>This is an DOCX test document, also made September 14, 2022</p>\n</body>" in document.content
        assert document.created is None

    def test_r_metadata_from_docx_plain(self, tika_client: TikaClient, sample_google_docs_to_docx_file: Path):
        documents = tika_client.rmeta.as_text.from_file(
            sample_google_docs_to_docx_file,
            magic.from_file(str(sample_google_docs_to_docx_file), mime=True),
        )

        assert len(documents) == 1
        document = documents[0]

        assert document.type == "application/vnd.openxmlformats-officedocument.wordprocessingml.document"
        assert document.content is not None
        assert "This is an DOCX test document, also made September 14, 2022" in document.content
        assert document.created is None

    def test_r_meta_microsoft_word_docx(self, tika_client: TikaClient, sample_docx_file: Path):
        documents = tika_client.rmeta.as_html.from_file(
            sample_docx_file,
            magic.from_file(str(sample_docx_file), mime=True),
        )

        assert len(documents) == 1
        document = documents[0]

        assert document.type == "application/vnd.openxmlformats-officedocument.wordprocessingml.document"
        assert document.content is not None
        assert (
            "<body><p>This is a sample document, generated by Microsoft Office on Wednesday, May 17, 2023.</p>\n<p>It is in English.</p>\n</body>"  # noqa: E501
            in document.content
        )

    def test_r_metadata_from_odt(self, tika_client: TikaClient, sample_google_docs_to_libre_office_writer_file: Path):
        documents = tika_client.rmeta.as_html.from_file(
            sample_google_docs_to_libre_office_writer_file,
            magic.from_file(str(sample_google_docs_to_libre_office_writer_file), mime=True),
        )

        assert len(documents) == 2
        document = documents[0]

        assert document.type == "application/vnd.oasis.opendocument.text"
        assert document.content is not None
        assert "<body><p>This is an ODT test document, created September 14, 2022</p>\n</body>" in document.content
        assert document.created is None

    def test_r_metadata_from_odt_plain(
        self,
        tika_client: TikaClient,
        sample_google_docs_to_libre_office_writer_file: Path,
    ):
        documents = tika_client.rmeta.as_text.from_file(
            sample_google_docs_to_libre_office_writer_file,
            magic.from_file(str(sample_google_docs_to_libre_office_writer_file), mime=True),
        )

        assert len(documents) == 2

        document = documents[0]
        assert document.type == "application/vnd.oasis.opendocument.text"
        assert document.content is not None
        assert "This is an ODT test document, created September 14, 2022" in document.content

        document = documents[1]
        assert document.type == "image/png"

    def test_r_metadata_from_ods_plain(self, tika_client: TikaClient, sample_ods_file: Path):
        documents = tika_client.rmeta.as_text.from_file(
            sample_ods_file,
            magic.from_file(str(sample_ods_file), mime=True),
        )

        assert len(documents) == 2

        document = documents[0]
        assert document.content is not None
        assert "This is cell A1" in document.content
        assert "You sunk my battleship" in document.content

        document = documents[1]
        assert document.type == "image/png"

    def test_r_metadata_from_xlsx_plain(self, tika_client: TikaClient, sample_xlsx_file: Path):
        documents = tika_client.rmeta.as_text.from_file(
            sample_xlsx_file,
            magic.from_file(str(sample_xlsx_file), mime=True),
        )

        assert len(documents) == 1
        document = documents[0]

        assert document.content is not None
        assert "This is cell A1" in document.content
        assert "You sunk my battleship" in document.content



# SPDX-FileCopyrightText: 2023-present Trenton H <rda0128ou@mozmail.com>
#
# SPDX-License-Identifier: MPL-2.0

from __future__ import annotations

import logging
from typing import TYPE_CHECKING

from httpx import Client

from tika_client._resource_meta import Metadata
from tika_client._resource_recursive import Recursive
from tika_client._resource_tika import Tika

if TYPE_CHECKING:
    from types import TracebackType

    from tika_client._types import Self


class TikaClient:
    def __init__(
        self,
        *,
        tika_url: str,
        timeout: float = 30.0,
        log_level: int = logging.ERROR,
        compress: bool = False,
    ):
        # Configure the client
        self._client = Client(base_url=tika_url, timeout=timeout)

        # Set the log level
        logging.getLogger("httpx").setLevel(log_level)
        logging.getLogger("httpcore").setLevel(log_level)

        # Only JSON responses supported
        self._client.headers.update({"Accept": "application/json"})

        if compress:
            self._client.headers.update({"Accept-Encoding": "gzip"})

        # Add the resources
        self.metadata = Metadata(self._client, compress=compress)
        self.tika = Tika(self._client, compress=compress)
        self.rmeta = Recursive(self._client, compress=compress)

    def add_headers(self, header: dict[str, str]) -> None:  # pragma: no cover
        """
        Updates the httpx Client headers with the given values
        """
        self._client.headers.update(header)

    def __enter__(self) -> Self:
        return self

    def __exit__(
        self,
        exc_type: type[BaseException] | None,
        exc_val: BaseException | None,
        exc_tb: TracebackType | None,
    ) -> None:
        self._client.close()



from datetime import datetime
from datetime import timezone
from pathlib import Path

import httpx
import magic
import pytest
from pytest_httpx import HTTPXMock

from tika_client.client import TikaClient


class TestMetadataResource:
    def test_metadata_from_docx(self, tika_client: TikaClient, sample_google_docs_to_docx_file: Path):
        """
        Test parsing of a DOCX produced by Google Docs conversion
        """

        resp = tika_client.metadata.from_file(
            sample_google_docs_to_docx_file,
            magic.from_file(str(sample_google_docs_to_docx_file), mime=True),
        )

        assert resp.type == "application/vnd.openxmlformats-officedocument.wordprocessingml.document"
        assert resp.created is None

    def test_metadata_from_docx_no_mime(self, tika_client: TikaClient, sample_google_docs_to_docx_file: Path):
        """
        Test parsing of a DOCX produced by Google Docs conversion, when no mime type is provided
        """

        resp = tika_client.metadata.from_file(sample_google_docs_to_docx_file)

        assert resp.type == "application/vnd.openxmlformats-officedocument.wordprocessingml.document"
        assert resp.created is None

    def test_metadata_from_word_docx(self, tika_client: TikaClient, sample_docx_file: Path):
        """
        Test parsing of a DOCX produced by Microsoft Word
        """
        resp = tika_client.metadata.from_file(sample_docx_file, magic.from_file(str(sample_docx_file), mime=True))

        assert resp.type == "application/vnd.openxmlformats-officedocument.wordprocessingml.document"
        assert resp.created == datetime(year=2023, month=5, day=17, hour=16, minute=41, tzinfo=timezone.utc)
        assert resp.modified == datetime(year=2023, month=5, day=17, hour=16, minute=44, tzinfo=timezone.utc)

    def test_metadata_from_odt(self, tika_client: TikaClient, sample_google_docs_to_libre_office_writer_file: Path):
        """
        Test parsing of a ODT produced by Google Docs conversion
        """
        resp = tika_client.metadata.from_file(
            sample_google_docs_to_libre_office_writer_file,
            magic.from_file(str(sample_google_docs_to_libre_office_writer_file), mime=True),
        )

        assert resp.type == "application/vnd.oasis.opendocument.text"
        assert resp.data["generator"] == "LibreOfficeDev/6.0.5.2$Linux_X86_64 LibreOffice_project/"
        assert resp.created is None

    def test_metadata_from_doc(self, tika_client: TikaClient, sample_doc_file: Path):
        """
        Test parsing of a DOC produced by Google Docs conversion
        """
        resp = tika_client.metadata.from_file(sample_doc_file, magic.from_file(str(sample_doc_file), mime=True))

        assert resp.type == "application/msword"
        assert resp.language == "en"

    def test_http_error(
        self,
        httpx_mock: HTTPXMock,
        tika_host: str,
        sample_google_docs_to_libre_office_writer_file: Path,
    ):
        """
        Test handling of HTTP errors returned from Tika
        """

        httpx_mock.add_response(status_code=500)
        with pytest.raises(httpx.HTTPStatusError) as err, TikaClient(tika_url=tika_host) as client:
            client.metadata.from_file(sample_google_docs_to_libre_office_writer_file)
        assert err.value.response.status_code == httpx.codes.INTERNAL_SERVER_ERROR



# SPDX-FileCopyrightText: 2023-present Trenton H <rda0128ou@mozmail.com>
#
# SPDX-License-Identifier: MPL-2.0

from __future__ import annotations

import sys
from typing import Optional
from typing import Union

if sys.version_info >= (3, 11):  # pragma: no cover
    from typing import Self
else:  # pragma: no cover
    from typing_extensions import Self  # noqa: F401

MimeType = Optional[str]

RequestContent = Union[str, bytes]



from pathlib import Path

import magic

from tika_client.client import TikaClient


class TestParseImageMetadata:
    def test_image_jpeg(self, tika_client: TikaClient, sample_jpeg_file: Path):
        """
        Test the handling of a JPEG file metadata retrieval
        """
        resp = tika_client.metadata.from_file(sample_jpeg_file, magic.from_file(str(sample_jpeg_file), mime=True))

        assert resp.type == "image/jpeg"

    def test_image_png(self, tika_client: TikaClient, sample_png_file: Path):
        """
        Test the handling of a PNG file metadata retrieval
        """
        resp = tika_client.metadata.from_file(sample_png_file, magic.from_file(str(sample_png_file), mime=True))

        assert resp.type == "image/png"



import shutil
from pathlib import Path

import magic

from tika_client.client import TikaClient


class TestParseFormatted:
    """
    Test the Tika endpoint for returning HTML formatted content
    """

    def test_parse_docx_from_file_as_html(self, tika_client: TikaClient, sample_google_docs_to_docx_file: Path):
        resp = tika_client.tika.as_html.from_file(
            sample_google_docs_to_docx_file,
            magic.from_file(str(sample_google_docs_to_docx_file), mime=True),
        )

        assert resp.type == "application/vnd.openxmlformats-officedocument.wordprocessingml.document"
        assert resp.content is not None
        assert "<body><p>This is an DOCX test document, also made September 14, 2022</p>\n</body>" in resp.content
        assert resp.content_length == 6424

    def test_parse_doc_from_file_as_html(self, tika_client: TikaClient, sample_doc_file: Path):
        resp = tika_client.tika.as_html.from_file(sample_doc_file, magic.from_file(str(sample_doc_file), mime=True))

        assert resp.type == "application/msword"
        assert resp.content is not None
        assert (
            "body><p>This is a test document, saved in the older .doc format for Word documents (but created in Google Drive)</p>\n</body>"  # noqa: E501
            in resp.content
        )
        assert resp.content_length == 23739
        assert resp.character_count == 90
        assert resp.page_count == 1
        assert resp.revision == 1
        assert resp.last_author == "cloudconvert_4"

    def test_parse_docx_no_mime_from_file_as_html(self, tika_client: TikaClient, sample_google_docs_to_docx_file: Path):
        resp = tika_client.tika.as_html.from_file(sample_google_docs_to_docx_file)

        assert resp.type == "application/vnd.openxmlformats-officedocument.wordprocessingml.document"
        assert resp.content is not None
        assert "<body><p>This is an DOCX test document, also made September 14, 2022</p>\n</body>" in resp.content

    def test_parse_microsoft_word_docx_from_file_as_html(self, tika_client: TikaClient, sample_docx_file: Path):
        resp = tika_client.tika.as_html.from_file(sample_docx_file, magic.from_file(str(sample_docx_file), mime=True))

        assert resp.type == "application/vnd.openxmlformats-officedocument.wordprocessingml.document"
        assert resp.content is not None
        assert (
            "<body><p>This is a sample document, generated by Microsoft Office on Wednesday, May 17, 2023.</p>\n<p>It is in English.</p>\n</body>"  # noqa: E501
            in resp.content
        )

    def test_parse_odt_from_file_as_html(
        self,
        tika_client: TikaClient,
        sample_google_docs_to_libre_office_writer_file: Path,
    ):
        resp = tika_client.tika.as_html.from_file(
            sample_google_docs_to_libre_office_writer_file,
            magic.from_file(str(sample_google_docs_to_libre_office_writer_file), mime=True),
        )

        assert resp.type == "application/vnd.oasis.opendocument.text"
        assert resp.content is not None
        assert "<body><p>This is an ODT test document, created September 14, 2022</p>\n</body>" in resp.content

    def test_parse_docx_from_buffer_as_html(self, tika_client: TikaClient, sample_google_docs_to_docx_file: Path):
        content = sample_google_docs_to_docx_file.read_bytes()
        resp = tika_client.tika.as_html.from_buffer(content, magic.from_buffer(content, mime=True))

        assert resp.type == "application/vnd.openxmlformats-officedocument.wordprocessingml.document"
        assert resp.content is not None
        assert "<body><p>This is an DOCX test document, also made September 14, 2022</p>\n</body>" in resp.content
        assert resp.content_length == 6183


class TestParsePlain:
    def test_parse_docx_from_file_as_text(self, tika_client: TikaClient, sample_google_docs_to_docx_file: Path):
        resp = tika_client.tika.as_text.from_file(
            sample_google_docs_to_docx_file,
            magic.from_file(str(sample_google_docs_to_docx_file), mime=True),
        )

        assert resp.type == "application/vnd.openxmlformats-officedocument.wordprocessingml.document"
        assert resp.content is not None
        assert "This is an DOCX test document, also made September 14, 2022" in resp.content

    def test_parse_odt_from_file_as_text(
        self,
        tika_client: TikaClient,
        sample_google_docs_to_libre_office_writer_file: Path,
    ):
        resp = tika_client.tika.as_text.from_file(
            sample_google_docs_to_libre_office_writer_file,
            magic.from_file(str(sample_google_docs_to_libre_office_writer_file), mime=True),
        )

        assert resp.type == "application/vnd.oasis.opendocument.text"
        assert resp.content is not None
        assert "This is an ODT test document, created September 14, 2022" in resp.content


class TestParseContentPlain:
    def test_parse_docx_from_bytes_buffer(self, tika_client: TikaClient, sample_google_docs_to_docx_file: Path):
        buffer = sample_google_docs_to_docx_file.read_bytes()

        resp = tika_client.tika.as_text.from_buffer(buffer)

        assert resp.type == "application/vnd.openxmlformats-officedocument.wordprocessingml.document"
        assert resp.content is not None
        assert "This is an DOCX test document, also made September 14, 2022" in resp.content

    def test_parse_odt_from_bytes_buffer(
        self,
        tika_client: TikaClient,
        sample_google_docs_to_libre_office_writer_file: Path,
    ):
        buffer = sample_google_docs_to_libre_office_writer_file.read_bytes()

        resp = tika_client.tika.as_text.from_buffer(buffer)

        assert resp.type == "application/vnd.oasis.opendocument.text"
        assert resp.content is not None
        assert "This is an ODT test document, created September 14, 2022" in resp.content

    def test_parse_odt_from_bytes_buffer_with_mime(
        self,
        tika_client: TikaClient,
        sample_google_docs_to_libre_office_writer_file: Path,
    ):
        buffer = sample_google_docs_to_libre_office_writer_file.read_bytes()

        resp = tika_client.tika.as_text.from_buffer(buffer, "application/vnd.oasis.opendocument.text")

        assert resp.type == "application/vnd.oasis.opendocument.text"
        assert resp.content is not None
        assert "This is an ODT test document, created September 14, 2022" in resp.content

    def test_html_document_from_string_buffer(self, tika_client: TikaClient, sample_html_file: Path):
        buffer = sample_html_file.read_text()

        resp = tika_client.tika.as_text.from_buffer(buffer)

        assert resp.type == "text/html; charset=UTF-8"
        assert resp.parsers == ["org.apache.tika.parser.DefaultParser", "org.apache.tika.parser.html.HtmlParser"]
        assert "Hello world! This is HTML5 content in a file for" in resp.data["X-TIKA:content"]
        assert resp.data["dc:title"] == "This Is A Test"
        assert resp.data["description"] == "A sample HTML file"


class TestParseContentCompress:
    def test_parse_odt_from_bytes_buffer_compress(
        self,
        tika_client_compressed: TikaClient,
        sample_office_doc_with_images_file: Path,
    ):
        buffer = sample_office_doc_with_images_file.read_bytes()

        resp = tika_client_compressed.tika.as_text.from_buffer(buffer)

        assert resp.type == "application/vnd.oasis.opendocument.text"


class TestFilenameContentDisposition:
    def test_non_ascii_filename(self, tika_client: TikaClient, sample_google_docs_to_docx_file: Path, tmp_path: Path):
        copy = shutil.copy(
            sample_google_docs_to_docx_file,
            tmp_path / "Kostenerstattung für Meldebescheinigung Familienzuschlag.docx",
        )

        resp = tika_client.tika.as_text.from_file(copy, magic.from_file(str(copy), mime=True))

        assert resp.type == "application/vnd.openxmlformats-officedocument.wordprocessingml.document"
        assert resp.content is not None
        assert "This is an DOCX test document, also made September 14, 2022" in resp.content



# SPDX-FileCopyrightText: 2023-present Trenton H <rda0128ou@mozmail.com>
#
# SPDX-License-Identifier: MPL-2.0

from tika_client.client import TikaClient
from tika_client.data_models import DublinCoreKey
from tika_client.data_models import TikaKey
from tika_client.data_models import XmpKey

__all__ = ["TikaClient", "TikaKey", "XmpKey", "DublinCoreKey"]



from datetime import datetime
from pathlib import Path

import magic

from tika_client.client import TikaClient


class TestLibreOfficeFormats:
    def test_parse_libre_office_writer_document(self, tika_client: TikaClient, sample_libre_office_writer_file: Path):
        """
        Test handling of a ODT document produced by LibreOffice
        """
        resp = tika_client.tika.as_html.from_file(
            sample_libre_office_writer_file,
            magic.from_file(str(sample_libre_office_writer_file), mime=True),
        )

        assert resp.type == "application/vnd.oasis.opendocument.text"
        assert resp.content is not None
        assert (
            "<body><p>This is a document created by LibreOffice Writer 7.5.12, on July 19th, 2023</p>\n</body>"
            in resp.content
        )
        assert resp.content_length == 11149
        assert resp.created is not None
        assert resp.created == datetime(
            year=2023,
            month=7,
            day=19,
            hour=11,
            minute=30,
            second=44,
            microsecond=719000,
            tzinfo=None,
        )



# SPDX-FileCopyrightText: 2023-present Trenton H <rda0128ou@mozmail.com>
#
# SPDX-License-Identifier: MPL-2.0

from typing import Final

# Only compress content which is larger than this
MIN_COMPRESS_LEN: Final[int] = 1024
